<?php

class shibd_discofeed
{
    protected string $url_list = '/Shibboleth.sso/DiscoFeed';
    protected string $url_login = '/Shibboleth.sso/Login';

    protected string $lang = 'en';

    // caching of discofeed
    protected string $_sCachefile = 'discofeed.json';
    protected int $_iCacheTtl;

    protected array $aConfig = [];

    // ----------------------------------------------------------------------

    /**
     * Constructor
     * @param array $aConfig  config array; the following keys are required:
     *                        idps        array   IDP poositive list
     *                        return-url  string  return URL
     *                        cache-ttl   int     caching time in seconds
     *                        cachefile   string  caching filename
     * @param string $SELFURL
     */
    public function __construct(array $aConfig, string $SELFURL = '')
    {
        $SELFURL = $SELFURL ?: (
            isset($_SERVER['SERVER_NAME']) ? "https://" . $_SERVER['SERVER_NAME'] : ''
        );

        if(!$SELFURL) {
            die("ERROR: SELFURL is not set. \$_SERVER['SERVER_NAME'] is not available.");
        }

        $this->url_list = "$SELFURL$this->url_list";
        $this->url_login = "$SELFURL$this->url_login";

        $this->_sCachefile = dirname(__DIR__).'/'
            .($aConfig['cachefile']??'discofeed.json')
            ;
        $this->_iCacheTtl = $aConfig['cachettl']??60*60;

        $this->lang = $aConfig['lang']??'en';

        $this->aConfig = $aConfig;
    }

    /**
     * Get List if IDPs from cache file if possible 
     * or from Shibboleth Disco feed and write a cache file
     * @return array
     */
    function getAllIdps(): array
    {
        if (!file_exists($this->_sCachefile) || filemtime($this->_sCachefile) < time() - $this->_iCacheTtl) {
            // echo "DEBUG: IDP - reading from Shibboleth<br>";
            $aReturn = json_decode(file_get_contents($this->url_list), 1);
    
            if ($aReturn && is_array($aReturn)) {
                // echo "DEBUG: IDP - storing cache<br>";
                file_put_contents($this->_sCachefile, json_encode($aReturn));
            }
        } else {
            // echo "DEBUG: IDP - reading cache<br>";
            $aReturn = json_decode(file_get_contents($this->_sCachefile), 1);
        }
    
        return isset($aReturn) && is_array($aReturn) ? $aReturn : [];
    }

    /**
     * Get list of active IDPs
     * @return array
     */
    public function getIdps(): array
    {
        $aAllIdps = $this->getAllIdps();

        $aReturn = [];
        if (is_array($aAllIdps) && count($aAllIdps)) {
            foreach ($aAllIdps as $aEntry) {
                $sEntityId = $aEntry['entityID'];
                $sIdpDomain = parse_url($sEntityId, PHP_URL_HOST);
                $sIdpTld = preg_filter('/^.*?\.([^\.]+)$/', '$1', $sIdpDomain);
    
                if (in_array($sEntityId, $this->aConfig['idps'])) {
    
                    $idxText=0;
                    foreach($aEntry['DisplayNames'] as $i=>$aLangitem){
                        if($aLangitem['lang']==$this->lang){
                            $idxText=$i;
                        }
                    }

                    $sLabel = $aEntry['DisplayNames'][$idxText]['value'] ?? parse_url($sEntityId, PHP_URL_HOST);
                    $sDescription = $aEntry['Descriptions'][$idxText]['value'] ?? '';
                    $sKeywords = $aEntry['Keywords'][$idxText]['value'] ?? '';

                    $sImage = $aEntry['Logos'][1]['value'] ?? ($aEntry['Logos'][0]['value'] ?? '');
    
                    $sUrl = $this->url_login
                        . '?entityID='. urlencode($sEntityId) 
                        . "&target=" . urlencode($sEntityId) 
                        . "&target=" . urlencode($this->SELFURL.($aConfig['return-url']??''))
                        ;
    
                    $aReturn[] = array_merge([
                        '_label' => $sLabel,
                        '_description' => $sDescription,
                        '_keywords' => $sKeywords,
                        '_image' => $sImage,
                        '_url' => $sUrl,
                        '_idpdomain' => $sIdpDomain,
                        '_tld' => $sIdpTld,
                        ], 
                        $aEntry
                    );
                }
            }
        }
        return $aReturn;
    }
}